#!/bin/bash
set -e

sys_root_dir=null

function usage() {
    echo -e "Usage:\n$0 <sys_root_dir>"
}

function main() {
    if [ $# -le 0 ]; then
        usage
        exit 1
    fi

    sys_root_dir=$1

    if [ ! -d build ]; then
        mkdir -p build
    else
        rm -rf build/*
    fi

    cd build

   # Define all necessary variables
    HOST_C_COMPLIER="$ENV{CROSS_COMPILE}gcc"
    HOST_CXX_COMPLIER="g++"
    HOST_COMPLIER_FLAGS="-O2 --sysroot=${sys_root_dir}"
    NPU_DEV_SIM=0
    ES_AI_INFERENCE_DIR=$(pwd)/..
    NPU_SDK_TEST_SAMPLES=${ES_AI_INFERENCE_DIR}
    NPU_SDK_ESWIN_HOST=${ES_AI_INFERENCE_DIR}
    WIN2030_ESSDK_INCLUDE=${sys_root_dir}/usr/include/essdk
    NPU_SDK_ESWIN_COMMON=${ES_AI_INFERENCE_DIR}

    # Run CMake with the specified SYS_ROOT_DIR
    cmake .. \
        -DHOST_CXX_COMPLIER=${HOST_CXX_COMPLIER} \
        -DHOST_COMPLIER_FLAGS="${HOST_COMPLIER_FLAGS}" \
        -DNPU_DEV_SIM=${NPU_DEV_SIM} \
        -DNPU_SDK_TEST_SAMPLES=${NPU_SDK_TEST_SAMPLES} \
        -DNPU_SDK_ESWIN_HOST=${NPU_SDK_ESWIN_HOST} \
        -DWIN2030_ESSDK_INCLUDE=${WIN2030_ESSDK_INCLUDE} \
        -DNPU_SDK_ESWIN_COMMON=${NPU_SDK_ESWIN_COMMON}

    make -j32
}

main $@
