// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include "file_util.h"

static inline int hexval(char c)
{
    switch (c) {
        case '0' ... '9':
            c -= '0';
            break;

        case 'a' ... 'f':
            c -= 'a';
            c += 10;
            break;

        case 'A' ... 'F':
            c -= 'A';
            c += 10;
            break;
    }
    return c;
}

int str2int(char *str)
{
    int val = 0;
    char *p, *cp = str;
    int base = 10, neg = 1;

    if ((p = strstr(cp, "0x")) != NULL) {
        cp = p + 2;
        base = 16;
    }

    if ((p = strstr(cp, "-")) != NULL) {
        cp = p + 1;
        base = 10;
        neg = -1;
    }

    for (; *cp != 0;) {
        val *= base;
        val += hexval(*cp);
        cp++;
    }

    return val * neg;
}

void *readBinFile(const char *fname, size_t *size)
{
    void *map_va = nullptr;
    size_t map_size = 0;

    int fd = open(fname, O_RDONLY);
    if (fd < 0) {
        printf("ERROR: could not open file %s\n", fname);
        return nullptr;
    }

    struct stat statbuf;
    int err = fstat(fd, &statbuf);
    if (err < 0) {
        printf("ERROR: fail to call fstat() for %s.\n", fname);
        close(fd);
        return nullptr;
    }

    map_va = (void *)mmap(NULL, statbuf.st_size, PROT_READ, MAP_SHARED, fd, 0);
    if (map_va == MAP_FAILED) {
        printf("ERROR: Mapping Failed\n");
        close(fd);
        return nullptr;
    }

    map_size = statbuf.st_size;
    *size = map_size;
    close(fd);

    return map_va;
}

void dumpBuf(FILE *fd, unsigned char *buf, int len)
{
    int i;

    for (i = 0; i < len; i += 4) {
        if (i % 16 == 0) {
            fprintf(fd, "\n");
        }
        fprintf(fd, "%02x%02x%02x%02x,", buf[i], buf[i + 1], buf[i + 2], buf[i + 3]);
    }
    fprintf(fd, "\n");
}

void dmabuf_statistic_check(void)
{
    int ret = -1;
    ret = system("cat /sys/kernel/debug/dma_buf/bufinfo");
    sleep(3);
}

int parseConfigFile(const char *file) { return 0; }

MODEL_DATA_TYPE_E parseModelDataType(const char *modelPath)
{
    if (strstr(modelPath, "fp32") != NULL) {
        printf("model data is fp32!\n");
        return OP_FP32;
    } else if (strstr(modelPath, "fp16") != NULL) {
        printf("model data is fp16!\n");
        return OP_FP16;
    } else {
        return OP_U8;
    }
}

float float16ToFloat(short *pfltInt16)
{
    float fRet;
    short fltInt16;
    if (pfltInt16 == NULL) {
        printf("pfltInt16 is NULL!\n");
        exit(-1);
    }

    fltInt16 = *pfltInt16;
    int fltInt32 = ((fltInt16 & 0x8000) << 16);
    fltInt32 |= ((fltInt16 & 0x7fff) << 13) + 0x38000000;
    memcpy(&fRet, &fltInt32, sizeof(float));

    return fRet;
}

bool isDirectoryExists(const char *path)
{
    struct stat info;
    if (stat(path, &info) != 0) return false;
    return S_ISDIR(info.st_mode);
}

bool isFileExists(const char *filename)
{
    struct stat buffer;
    return (stat(filename, &buffer) == 0);
}
