// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#include "log.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <pthread.h>

#define THREAD_ID pthread_self()

#define PRINT_SIZE 1024
#define TIMESTAMP_SIZE 32

es_log_level SET_LEVEL = LOG_LEVEL;

static void get_timestamp(char* buffer)
{
    int len;
    int millsec;

    struct timeval tv = {0};
    gettimeofday(&tv, NULL);
    millsec = (int)(tv.tv_usec / 1000);

    time_t time_seconds = time(0);
    struct tm now;
    localtime_r(&time_seconds, &now);

    len = snprintf(buffer, TIMESTAMP_SIZE, "%02d:%02d:%02d.%03d", now.tm_hour, now.tm_min, now.tm_sec, millsec);

    buffer[len] = '\0';
}

void es_set_level(es_log_level level) { SET_LEVEL = level; }

void es_log(const char* module, const char* func, int line, es_log_level level, const char* fmt, ...)
{
    if (level > LOG_LEVEL || level > SET_LEVEL) return;

    va_list arg_list;
    char buf[PRINT_SIZE];
    memset(buf, 0, sizeof(buf));
    va_start(arg_list, fmt);
    vsnprintf(buf, sizeof(buf), fmt, arg_list);

    const char* tmp = strrchr(func, '/');
    if (!tmp)
        tmp = func;
    else
        tmp++;

    char time[TIMESTAMP_SIZE] = {0};
    get_timestamp(time);

    switch (level) {
        case DEBUG_LEVEL:
            printf("[%s][\033[1;34mDEBUG\033[0m][T:0x%lx](%s:%d) %s", time, THREAD_ID, tmp, line, buf);
            break;
        case INFO_LEVEL:
            printf("[%s][\033[1;32mINFO\033[0m][T:0x%lx](%s:%d) %s", time, THREAD_ID, tmp, line, buf);
            break;
        case WARN_LEVEL:
            // yellow
            printf("[%s][\033[1;33mWARN\033[0m][T:0x%lx](%s:%d) %s", time, THREAD_ID, tmp, line, buf);
            break;
        case ERROR_LEVEL:
            // red
            printf("[%s][\033[1;31mERROR\033[0m][T:0x%lx](%s:%d) %s", time, THREAD_ID, tmp, line, buf);
            break;
        default:
            break;
    }

    va_end(arg_list);
}
