// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESWIN_COMMON_LOGGER_LOG_H_
#define ESWIN_COMMON_LOGGER_LOG_H_

#include "hetero_types.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LOG_LEVEL
#define LOG_LEVEL ERROR_LEVEL
#endif

#define ERROR_LEVEL 1U
#define WARN_LEVEL 2U
#define INFO_LEVEL 3U
#define DEBUG_LEVEL 4U
typedef u32 es_log_level;

void es_log(const char* module, const char* func, int line, es_log_level level, const char* fmt, ...);
void es_set_level(es_log_level level);

#if !defined(LOG_TAG)
#define LOG_TAG "NO_TAG"
#endif

#define ES_LOG_SET_LEVEL(level) es_set_level(level)
#define LOG_DBG(format, ...) es_log(LOG_TAG, __func__, __LINE__, DEBUG_LEVEL, format, ##__VA_ARGS__)
#define LOG_MSG(format, ...) es_log(LOG_TAG, __func__, __LINE__, INFO_LEVEL, format, ##__VA_ARGS__)
#define LOG_WARN(format, ...) es_log(LOG_TAG, __func__, __LINE__, WARN_LEVEL, format, ##__VA_ARGS__)
#define LOG_ERR(format, ...) es_log(LOG_TAG, __func__, __LINE__, ERROR_LEVEL, format, ##__VA_ARGS__)

#define LOG_DBG_T(tag, format, ...) es_log(tag, __func__, __LINE__, DEBUG_LEVEL, format, ##__VA_ARGS__)
#define LOG_MSG_T(tag, format, ...) es_log(tag, __func__, __LINE__, INFO_LEVEL, format, ##__VA_ARGS__)
#define LOG_WARN_T(tag, format, ...) es_log(tag, __func__, __LINE__, WARN_LEVEL, format, ##__VA_ARGS__)
#define LOG_ERR_T(tag, format, ...) es_log(tag, __func__, __LINE__, ERROR_LEVEL, format, ##__VA_ARGS__)

#ifdef __cplusplus
};
#endif

#endif  // ESWIN_COMMON_LOGGER_LOG_H_
