// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_DSPPROCESSOR_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_DSPPROCESSOR_H_

#include "iHWProcessor.h"
#include "opThreadScheduler.h"
#include "es_dsp_types.h"

#define DSP_MAX_CORE_NUM 4

namespace eswin
{

///////////////////////////////////////////////////////////////////////////////
// Class DspProcessor implementation
class DspProcessor : public IHWProcessor
{
 public:
    explicit DspProcessor(int32_t devId);
    virtual ~DspProcessor();
    int32_t open() override;
    int32_t close() override;

    int32_t processReport() override;
    int32_t getFd() override;
    int32_t submitOpTask(opTaskDesc_t* opTaskDesc) override;
    int32_t submitModelTask(DSP_TASK_DESC_S* desc, int32_t numTask);

    int32_t dspLoadOp(const char* opName, const char* opLibDir, int32_t* opHandle);
    int32_t dspUnLoadOp(const int32_t opHandle);

    static void asyncTaskCb(void* arg, int32_t state);

    ES_S32 prepareDmaBuffer(ES_DEV_BUF_S buffer);
    ES_S32 unPrepareDmaBuffer(ES_U64 fd);

    ES_S32 enablePerf(bool enable);
    ES_S32 getPerfData(void* data);
    ES_S32 getHwPerfData(void* data);

 private:
    int32_t mFd;
    ES_DSP_ID_E mDevId;
};
}  // namespace eswin

#endif  // #ifndef ESWIN_HOST_RUNTIME_INCLUDE_DSPPROCESSOR_H_
