// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_EDLA_LOG_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_EDLA_LOG_H_

#include <string>
#include <cstdint>
#include "es_npu_error.h"
#include "es_npu_types.h"

#if NPU_PERF_STATS > 1
#define CURRENT_LEVEL EDLA_LOG_LEVEL_ERROR
#define EDLA_LOG_WARN(...)
#define EDLA_LOG_INFO(...)
#define EDLA_LOG_DEBUG(...)
#define EDLA_LOG_DETAIL(...)
#else
#if LOG_LEVEL == 0
#define CURRENT_LEVEL EDLA_LOG_LEVEL_ERROR
#elif LOG_LEVEL == 1
#define CURRENT_LEVEL EDLA_LOG_LEVEL_WARN
#elif LOG_LEVEL == 2
#define CURRENT_LEVEL EDLA_LOG_LEVEL_INFO
#else
#define CURRENT_LEVEL EDLA_LOG_LEVEL_DEBUG
#endif
#if (NPU_DEV_SIM == NPU_REAL_ENV)
#include "es_log.h"
#define EDLA_LOG_WARN(fmt, args...) es_log_print(ES_ID_NPU, ES_LOG_WARN, fmt, ##args)
#define EDLA_LOG_INFO(fmt, args...) es_log_print(ES_ID_NPU, ES_LOG_NOTICE, fmt, ##args)
#define EDLA_LOG_DEBUG(fmt, args...) es_log_print(ES_ID_NPU, ES_LOG_INFO, fmt, ##args)
#define EDLA_LOG_DETAIL(fmt, args...) es_log_print(ES_ID_NPU, ES_LOG_DBG, fmt, ##args)
#else
#define EDLA_LOG_WARN(...) EdlaLog::GetInstance().Print(EDLA_LOG_LEVEL_WARN, __func__, __LINE__, ##__VA_ARGS__)
#define EDLA_LOG_INFO(...) EdlaLog::GetInstance().Print(EDLA_LOG_LEVEL_INFO, __func__, __LINE__, ##__VA_ARGS__)
#define EDLA_LOG_DEBUG(...) EdlaLog::GetInstance().Print(EDLA_LOG_LEVEL_DEBUG, __func__, __LINE__, ##__VA_ARGS__)
#define EDLA_LOG_DETAIL(...) EdlaLog::GetInstance().Print(EDLA_LOG_LEVEL_DETAIL, __func__, __LINE__, ##__VA_ARGS__)
#endif
#endif
#if (NPU_DEV_SIM == NPU_REAL_ENV)
#include "es_log.h"
#define EDLA_LOG_ERR(fmt, args...) es_log_print(ES_ID_NPU, ES_LOG_ERR, fmt, ##args)
#define EDLA_LOG_SET_LEVEL(Level) es_log_set_level(ES_ID_NPU, (es_log_level)Level)
#else
#define EDLA_LOG_ERR(...) EdlaLog::GetInstance().Print(EDLA_LOG_LEVEL_ERROR, __func__, __LINE__, ##__VA_ARGS__)
#define EDLA_LOG_SET_LEVEL(Level) EdlaLog::GetInstance().SetLevel((EDLA_LOG_LEVEL_E)Level);
#endif

#define LOG_MAX_LEN 512

typedef enum {
    EDLA_LOG_LEVEL_ERROR = 0,
    EDLA_LOG_LEVEL_WARN,
    EDLA_LOG_LEVEL_INFO,
    EDLA_LOG_LEVEL_DEBUG,
    EDLA_LOG_LEVEL_DETAIL,
} EDLA_LOG_LEVEL_E;

class EdlaLog
{
 private:
    EdlaLog();
    ~EdlaLog();

 public:
    static EdlaLog &GetInstance();
    int32_t Init();
    void DeInit();
    void Print(EDLA_LOG_LEVEL_E Level, const char *Func, uint32_t Line, const char *Format, ...);
    void SetLevel(EDLA_LOG_LEVEL_E Level);
    EDLA_LOG_LEVEL_E GetLevel();

 private:
    std::string mFilePath;
    std::string mFileName;
    int32_t mFd;
    EDLA_LOG_LEVEL_E mLevel;
    pthread_mutex_t mMutex;
};

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_EDLA_LOG_H_
