// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#ifndef ESWIN_HOST_RUNTIME_INCLUDE_ESTIMER_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_ESTIMER_H_

#include <iostream>
#include <thread>
#include <chrono>
#include <atomic>
#include <functional>
#include <condition_variable>
namespace eswin
{
class EsTimer
{
 public:
    EsTimer();
    ~EsTimer();
    void init(int32_t interval, std::function<int32_t(void *, void *, uint32_t, int32_t)> callback, void *arg1,
              void *arg2, uint32_t modelId, int32_t n);
    void start();
    void stop();
    void reset();
    bool isInited();

 private:
    std::thread mTimeThread;
    std::atomic<bool> mRunning;
    std::atomic<bool> mThreadExited;
    int32_t mTimeout;  // ms
    std::mutex mMutex;
    std::condition_variable mCv;
    std::function<int32_t(void *, void *, uint32_t, int32_t)> mCallback;
    void *mCallbackArg1;
    void *mCallbackArg2;
    void *threadFunc(uint32_t modelId, int32_t n);
};
};      // namespace eswin
#endif  // ESWIN_HOST_RUNTIME_INCLUDE_ESTIMER_H_