// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_HWPROCESSORMGR_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_HWPROCESSORMGR_H_

#include <map>
#include <array>
#include <memory>
#include <unordered_map>
#include "iHWProcessor.h"
#include "epollWrapper.h"
namespace eswin
{
#define DIE_NUM 2
typedef enum {
    DEV_TYPE_DSP0 = 0,
    DEV_TYPE_DSP1 = 1,
    DEV_TYPE_DSP2 = 2,
    DEV_TYPE_DSP3 = 3,
    DEV_TYPE_HAE = 4,
    DEV_TYPE_GPU = 5,
    DEV_TYPE_NPU = 6,
    DEV_TYPE_NONE,
} DEVICE_TYPE_E;

class HWProcessorMgr
{
 public:
    HWProcessorMgr();
    ~HWProcessorMgr();
    int32_t open(uint16_t dieId);
    int32_t close();

    std::shared_ptr<IHWProcessor> getProcessor(int32_t opType);
    std::shared_ptr<IHWProcessor> getProcessor(DEVICE_TYPE_E devType);
    DEVICE_TYPE_E getDeviceTypeByFd(int32_t fd);
    int32_t setContext(npu_context context);
    int32_t waitEvents();

 private:
    EpollWrapper mEpoll;
    std::map<int32_t, std::shared_ptr<IHWProcessor>> mProcessors;

    static const std::array<std::unordered_map<DEVICE_TYPE_E, int32_t>, DIE_NUM> mDevIdMapsArray;
};
}  // namespace eswin

#endif  // #ifndef ESWIN_HOST_RUNTIME_INCLUDE_HWPROCESSORMGR_H_