// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_ILOADABLEENTRY_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_ILOADABLEENTRY_H_

#include "loadableType.h"

#define ESWIN_LOADABLE_INTERFACE_NONE 0U
#define ESWIN_LOADABLE_INTERFACE_UMD 1U
#define ESWIN_LOADABLE_INTERFACE_KMD 2U

#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_NONE 0U
#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_ADDR0 1U
#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_DEPS 2U
#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_OPS 3U
#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_SURFS 4U
#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_LUTS 5U
#define ESWIN_LOADABLE_SUB_INTERFACE_DLA1_DSP 6U

#define ESWIN_LOADABLE_SUB_INTERFACE_EMU1_NONE 0U
#define ESWIN_LOADABLE_SUB_INTERFACE_EMU1_ADDR0 1U
/* #define ESWIN_LOADABLE_SUB_INTERFACE_EMU1_DEPS  2U */
#define ESWIN_LOADABLE_SUB_INTERFACE_EMU1_OPS 3U
#define ESWIN_LOADABLE_SUB_INTERFACE_EMU1_SURFS 4U
/* #define ESWIN_LOADABLE_SUB_INTERFACE_EMU1_LUTS  5U */

#define ESWIN_LOADABLE_SUB_INTERFACE_UMD_NONE 0U
#define ESWIN_LOADABLE_SUB_INTERFACE_UMD_ADDR0 1U
#define ESWIN_LOADABLE_SUB_INTERFACE_UMD_DEPS 2U
#define ESWIN_LOADABLE_SUB_INTERFACE_UMD_OPS 3U
#define ESWIN_LOADABLE_SUB_INTERFACE_UMD_SURFS 4U
#define ESWIN_LOADABLE_SUB_INTERFACE_UMD_CFGS 5U

#define ESWIN_LOADABLE_SUB_INTERFACE_2D_NONE 0U
#define ESWIN_LOADABLE_SUB_INTERFACE_2D_ADDR0 1U
#define ESWIN_LOADABLE_SUB_INTERFACE_2D_DEPS 2U
#define ESWIN_LOADABLE_SUB_INTERFACE_2D_OPS 3U
#define ESWIN_LOADABLE_SUB_INTERFACE_2D_SURFS 4U

#define ESWIN_LOADABLE_MEMORY_DOMAIN_SYSMEM 0U
#define ESWIN_LOADABLE_MEMORY_DOMAIN_SRAM 1U

#define ESWIN_LOADABLE_MEMORY_FLAGS_NONE 0U
#define ESWIN_LOADABLE_MEMORY_FLAGS_ALLOC 1U
#define ESWIN_LOADABLE_MEMORY_FLAGS_SET 2U
#define ESWIN_LOADABLE_MEMORY_FLAGS_INPUT 4U
#define ESWIN_LOADABLE_MEMORY_FLAGS_OUTPUT 8U
#define ESWIN_LOADABLE_MEMORY_FLAGS_DEBUG 16U
#define ESWIN_LOADABLE_MEMORY_FLAGS_CACHE 32U
#define ESWIN_LOADABLE_MEMORY_FLAGS_LLC 64U

#define ESWIN_LOADABLE_EVENT_OP_WAIT 0U
#define ESWIN_LOADABLE_EVENT_OP_SIGNAL 1U

#define ESWIN_LOADABLE_TENSOR_DESC_NUM_STRIDES 8U /* a little room to grow */

struct esLoadableI;
typedef struct esLoadable {
    void *self;
    const struct esLoadableI *i;
} esLoadable;

typedef struct esLoadableVersion {
    uint8_t major;
    uint8_t minor;
    uint8_t subMinor;
} esLoadableVersion;

typedef struct esLoadableMemoryListEntry {
    uint16_t id;
    uint64_t size;
    uint32_t alignment;
    uint8_t domain;
    uint8_t flags;
    uint16_t bindId;
    uint16_t tensorDescId;
    size_t numContents;
    uint8_t **contents;
    uint32_t *offsets;
} esLoadableMemoryListEntry;

typedef struct esLoadableEventListEntry {
    uint16_t id;
    uint16_t target;
    uint8_t op;
    uint32_t val;
} esLoadableEventListEntry;

typedef struct esLoadableSubModelListEntry {
    uint16_t id;
    uint32_t interface;
    int16_t kmdId;
    uint16_t numAddressList;
    uint16_t *addressList;  // [addr list id]...[addr list id]
} esLoadableSubModelListEntry;

typedef struct esLoadableAddressListEntry {
    uint16_t id;
    uint16_t memId;
    uint64_t size;
    uint64_t offset;
} esLoadableAddressListEntry;

typedef struct esLoadableBlob {
    char *name;
    uint64_t size;
    uint32_t interface;
    uint32_t sub_interface;
    esLoadableVersion version;
} esLoadableBlob;

struct esLoadableI {
    const char *(*getName)(esLoadable);
};

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_ILOADABLEENTRY_H_
