// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_KMDINTERFACE_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_KMDINTERFACE_H_

#include <linux/ioctl.h>
#include <linux/types.h>
#include "es_npu_error.h"
#include "es_npu_types.h"
#include "hetero_ioctl.h"
#include "core.h"
#include "task.h"

namespace eswin
{

struct taskResult_t {
    uint32_t taskId;
    uint32_t status;
};

struct esTaskIoctl {
    uint32_t taskId;
    uint32_t modelId;
    uint8_t inputFdNum;
    uint8_t outputFdNum;
    uint32_t inputFd[ES_TASK_MAX_FD_CNT];
    uint32_t outputFd[ES_TASK_MAX_FD_CNT];
    uint32_t status;
};

struct taskRec {
    uint32_t streamId;
    uint32_t taskNum;
    uint64_t taskAddr;
};

struct streamRec {
    uint32_t stramId;
    uint64_t bufVirtAddr;
};

#define ESWIN_MAX_BUFFERS_PER_TASK (6144)

enum ADDRESS_LIST_FLAG {
    FLAG_INPUT = 0xfe,
    FLAG_OUTPUT,
};

int32_t submitModelToKmd(uint32_t devFd, modelRec_t *mModelRec, uint32_t *modelId);
int32_t unloadModelFromKmd(uint32_t devFd, modelRec_t *mModelRec, uint32_t modelId);
int32_t submitTask(uint32_t devFd, uint32_t streamId, uint32_t sync, npuTask_t *tasks, uint32_t numTasks);
int32_t createKmdStream(uint32_t devFd, uint32_t *streamId);
int32_t synchronizeKmdStream(uint32_t devFd, uint32_t streamId);
int32_t abortKmdStream(uint32_t devFd, uint32_t streamId);
int32_t dstroyKmdStream(uint32_t devFd, uint32_t streamId);
int32_t getTaskResult(uint32_t devFd, uint32_t streamId, uint32_t modelId, uint32_t blockMode,
                      struct tasks_result_t *tasks_result);
}  // namespace eswin
int32_t esAllocDmaBufMem(void **pData, int32_t cache_type, const char *strZone, uint64_t len);
int32_t esFreeDmaBufMem(int32_t fd, void *pData, uint64_t len);

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_KMDINTERFACE_H_
