// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_LOADABLETYPE_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_LOADABLETYPE_H_

#include <cstdint>
#include "es_npu_types.h"

#define ESWIN_DATA_FORMAT_UNKNOWN 0U
#define ESWIN_DATA_FORMAT_NCHW 1U
#define ESWIN_DATA_FORMAT_NHWC 2U
#define ESWIN_DATA_FORMAT_NCxHWx 3U

#define ESWIN_DATA_TYPE_UNKNOWN 0U
#define ESWIN_DATA_TYPE_FLOAT 1U
#define ESWIN_DATA_TYPE_HALF 2U
#define ESWIN_DATA_TYPE_INT16 3U
#define ESWIN_DATA_TYPE_INT8 4U
#define ESWIN_DATA_TYPE_UINT8 5U
#define ESWIN_DATA_TYPE_UINT16 6U

#define ESWIN_TENSOR_SCALING_MODE_NONE 0U
#define ESWIN_TENSOR_SCALING_MODE_PER_TENSOR 1U
#define ESWIN_TENSOR_SCALING_MODE_PER_CHANNEL 2U

#define ESWIN_TENSOR_QUANTIZATION_MODE_NONE 0U
#define ESWIN_TENSOR_QUANTIZATION_MODE_PER_KERNEL 1U
#define ESWIN_TENSOR_QUANTIZATION_MODE_PER_FILTER 2U

#define ESWIN_DATA_CATEGORY_IMAGE 0U
#define ESWIN_DATA_CATEGORY_WEIGHT 1U
#define ESWIN_DATA_CATEGORY_FEATURE 2U
#define ESWIN_DATA_CATEGORY_PLANAR 3U
#define ESWIN_DATA_CATEGORY_BIAS 4U

#define ESWIN_WEIGHT_PACKING_UNKNOWN 0U
#define ESWIN_WEIGHT_PACKING_UNCOMPRESSED 1U
#define ESWIN_WEIGHT_PACKING_COMPRESSED 2U

#define ESWIN_PIXEL_FORMAT_R8 0U
#define ESWIN_PIXEL_FORMAT_R10 1U
#define ESWIN_PIXEL_FORMAT_R12 2U
#define ESWIN_PIXEL_FORMAT_R16 3U
#define ESWIN_PIXEL_FORMAT_R16_I 4U
#define ESWIN_PIXEL_FORMAT_R16_F 5U
#define ESWIN_PIXEL_FORMAT_A16B16G16R16 6U
#define ESWIN_PIXEL_FORMAT_X16B16G16R16 7U
#define ESWIN_PIXEL_FORMAT_A16B16G16R16_F 8U
#define ESWIN_PIXEL_FORMAT_A16Y16U16V16 9U
#define ESWIN_PIXEL_FORMAT_V16U16Y16A16 10U
#define ESWIN_PIXEL_FORMAT_A16Y16U16V16_F 11U
#define ESWIN_PIXEL_FORMAT_A8B8G8R8 12U
#define ESWIN_PIXEL_FORMAT_A8R8G8B8 13U
#define ESWIN_PIXEL_FORMAT_B8G8R8A8 14U
#define ESWIN_PIXEL_FORMAT_R8G8B8A8 15U
#define ESWIN_PIXEL_FORMAT_X8B8G8R8 16U
#define ESWIN_PIXEL_FORMAT_X8R8G8B8 17U
#define ESWIN_PIXEL_FORMAT_B8G8R8X8 18U
#define ESWIN_PIXEL_FORMAT_R8G8B8X8 19U
#define ESWIN_PIXEL_FORMAT_A2B10G10R10 20U
#define ESWIN_PIXEL_FORMAT_A2R10G10B10 21U
#define ESWIN_PIXEL_FORMAT_B10G10R10A2 22U
#define ESWIN_PIXEL_FORMAT_R10G10B10A2 23U
#define ESWIN_PIXEL_FORMAT_A2Y10U10V10 24U
#define ESWIN_PIXEL_FORMAT_V10U10Y10A2 25U
#define ESWIN_PIXEL_FORMAT_A8Y8U8V8 26U
#define ESWIN_PIXEL_FORMAT_V8U8Y8A8 27U
#define ESWIN_PIXEL_FORMAT_Y8___U8V8_N444 28U
#define ESWIN_PIXEL_FORMAT_Y8___V8U8_N444 29U
#define ESWIN_PIXEL_FORMAT_Y10___U10V10_N444 30U
#define ESWIN_PIXEL_FORMAT_Y10___V10U10_N444 31U
#define ESWIN_PIXEL_FORMAT_Y12___U12V12_N444 32U
#define ESWIN_PIXEL_FORMAT_Y12___V12U12_N444 33U
#define ESWIN_PIXEL_FORMAT_Y16___U16V16_N444 34U
#define ESWIN_PIXEL_FORMAT_Y16___V16U16_N444 35U
#define ESWIN_PIXEL_FORMAT_FEATURE 36U
#define ESWIN_PIXEL_FORMAT_FEATURE_X8 37U
#define ESWIN_PIXEL_FORMAT_UNKNOWN 37U

#define ESWIN_PIXEL_MAPPING_PITCH_LINEAR 0U

typedef struct esWeights {
    uint8_t type; /*_DATA_TYPE*/
    const void* values;
    int64_t count;
} esWeights;

#define ESWIN_LAYER_TYPE_CONVOLUTION 0U
#define ESWIN_LAYER_TYPE_FULLY_CONNECTED 1U
#define ESWIN_LAYER_TYPE_ACTIVATION 2U
#define ESWIN_LAYER_TYPE_POOLING 3U
#define ESWIN_LAYER_TYPE_LRN 4U
#define ESWIN_LAYER_TYPE_SCALE 5U
#define ESWIN_LAYER_TYPE_BATCH_NORM 6U
#define ESWIN_LAYER_TYPE_SOFTMAX 7U
#define ESWIN_LAYER_TYPE_DECONVOLUTION 8U
#define ESWIN_LAYER_TYPE_CONCATENATION 9U
#define ESWIN_LAYER_TYPE_ELEMENTWISE 10U
#define ESWIN_LAYER_TYPE_SLICE 11U
#define ESWIN_LAYER_TYPE_UNKNOWN 12U

#define ESWIN_ACTIVATION_TYPE_RELU 0U
#define ESWIN_ACTIVATION_TYPE_SIGMOID 1U
#define ESWIN_ACTIVATION_TYPE_TANH 2U

#define ESWIN_POOLING_TYPE_MIN 0U
#define ESWIN_POOLING_TYPE_MAX 1U
#define ESWIN_POOLING_TYPE_AVERAGE 2U

#define ESWIN_BIAS_MODE_NONE 0U
#define ESWIN_BIAS_MODE_UNIFORM 1U
#define ESWIN_BIAS_MODE_CHANNEL 2U
#define ESWIN_BIAS_MODE_ELEMENTWISE 3U

#define ESWIN_SCALE_MODE_UNKNOWN 0U
#define ESWIN_SCALE_MODE_UNIFORM 1U
#define ESWIN_SCALE_MODE_CHANNEL 2U
#define ESWIN_SCALE_MODE_ELEMENTWISE 3U

#define ESWIN_BATCH_NORM_MODE_UNIFORM 0U
#define ESWIN_BATCH_NORM_MODE_CHANNEL 1U

#define ESWIN_ELEMENTWISE_OPERATION_SUM 0U
#define ESWIN_ELEMENTWISE_OPERATION_PROD 1U
#define ESWIN_ELEMENTWISE_OPERATION_MIN 2U
#define ESWIN_ELEMENTWISE_OPERATION_MAX 3U

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_LOADABLETYPE_H_
