// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_NPUMEMPOOL_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_NPUMEMPOOL_H_
#include <iostream>
#include <memory>
#include <vector>
#include <mutex>
#include "es_npu_types.h"

#define DEFAULT_MEM_POOL_SIZE 192

namespace eswin
{
typedef struct {
    std::vector<ES_DEV_BUF_S> input;
    std::vector<ES_DEV_BUF_S> output;
    bool busy;
} MemBlock;

class NpuMemPool
{
 public:
    NpuMemPool();
    ~NpuMemPool();
    int32_t init(uint16_t devId, int32_t batchSize, int32_t size, std::vector<int32_t> inSizes,
                 std::vector<int32_t> outSizes, uint64_t &totalInputSize);
    void deinit();
    int32_t getMemBlock(std::shared_ptr<MemBlock> &block);
    int32_t releaseMemBlock(MemBlock &block);

 private:
    std::mutex mMtx;
    std::vector<std::shared_ptr<MemBlock>> mMemPool;
    std::vector<ES_U64> mInputFd;
    std::vector<uint64_t> mInputSize;
    std::vector<ES_U64> mOutputFd;
    std::vector<uint64_t> mOutputSize;
    int32_t mWritePos;
    int32_t mCapability;
    int32_t mSize;
};
}  // namespace eswin
#endif  // ESWIN_HOST_RUNTIME_INCLUDE_NPUMEMPOOL_H_