// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_NPUPROCESSOR_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_NPUPROCESSOR_H_

#include "core.h"
#include "iHWProcessor.h"
#include "es_npu_types.h"
#include "model.h"
namespace eswin
{
class NpuProcessor : public IHWProcessor
{
 public:
    explicit NpuProcessor(int32_t devId);
    virtual ~NpuProcessor();
    int32_t open() override;
    int32_t close() override;

    int32_t processReport() override;
    int32_t getFd() override;
    int32_t submitOpTask(opTaskDesc_t *opTaskDesc) override;

    int32_t npuSubmitModel(int32_t modelRecDmabufFd, uint16_t *modelId);
    int32_t npuSubmitTask(npuTask_t *task, const kmd_dump_info_t *dumpInfo, bool isDump);
    int32_t npuUnloadModel(uint16_t modelId);
    int32_t npuGetSram(int32_t *sramFd, uint32_t *sramLen);
    int32_t npuHandlePerf(bool enable);
    int32_t npuGetPerfData(void *dataBuf);
    int32_t npuPrepare(ES_DEV_BUF_S *devBuf);
    int32_t npuUnprepare(ES_DEV_BUF_S *devBuf);

 private:
    int32_t mFd;
    int32_t mDevId;
    const priv::UmdSubModel *mUmd;
    priv::esModel *mModel;
};
}  // namespace eswin

#endif  // #ifndef ESWIN_HOST_RUNTIME_INCLUDE_NPUPROCESSOR_H_
