// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_OPTHREADSCHEDULER_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_OPTHREADSCHEDULER_H_

#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <thread>
#include <queue>
#include <sys/eventfd.h>
#include <condition_variable>
#include <mutex>
#include "model.h"
#include "es_npu_error.h"
#include "iHWProcessor.h"

namespace eswin
{

class OpThreadScheduler
{
 public:
    OpThreadScheduler()
        : mDev(INVALID_HANDLE),
          mRunning(false),
          mRunThread(nullptr),
          mEventFd(INVALID_HANDLE),
          mReadyQueue{},
          mCompletedQueue{}
    {
    }

    ~OpThreadScheduler()
    {
        stopSchedule();
        if (mEventFd >= 0) {
            close(mEventFd);
        }
    }

    int32_t initialize(int32_t dev, npuContext_t *ctx);  // 初始化

    /*
     * 向驱动提交执行任务的同步函数
     */
    virtual int32_t submitOpTask(opTaskDesc_t *opTask) = 0;

    /*
     * 将task挂到待执行队列
     */
    int32_t queueOpTask(opTaskDesc_t *opTask);

    /*
     * 从完成队列取出每个opTask调用callback
     */
    int32_t processReport(void);
    int32_t getFd();  // 返回mEventFd

 private:
    void runThreadFunc(npuContext_t *ctx);     // 线程函数
    int32_t startSchedule(npuContext_t *ctx);  // 创建线程并运行
    void stopSchedule();                       // 停止线程运行并销毁

    std::queue<opTaskDesc_t *> mReadyQueue;      // 待执行队列
    std::queue<opTaskDesc_t *> mCompletedQueue;  // 已执行完成队列

    std::thread *mRunThread;
    std::atomic<bool> mRunning;

    std::mutex mCompletedMutex;
    std::mutex mReadyMutex;
    std::condition_variable mCond;

    int32_t mEventFd;

 protected:
    int32_t mDev;
};

}  // namespace eswin

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_OPTHREADSCHEDULER_H_
