// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_TASK_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_TASK_H_

#include <vector>
#include "es_npu_types.h"
#include "list.h"
#include "dla_interface.h"
#include "es_dsp_types.h"
#include "es2d_api.h"

#define ES_NPU_OP_INIT 0
#define ES_NPU_OP_SUBMIT 1
#define ES_NPU_OP_DONE 2
namespace eswin
{

typedef struct _esHaeTask {
    ES2D_SURFACE src;
    ES2D_SURFACE dst;
    ES2D_OP op;
    ES2D_NORMALIZATION_PARAMS para;
    ES2D_RECT *srcRect;
    ES2D_RECT *dstRect;
} esHaeTask_t;

typedef struct _dataDesc {
    /**
     * These structures can refer to the UMD
     */
    struct dla_network_desc *network;
    struct dla_common_op_desc *common_desc;
    union dla_surface_container *surface_desc;
    union dla_operation_container *op_desc;
} dataDesc_t;

typedef int32_t (*opTaskCallback)(void *arg);

typedef struct _opPrivate {
    opTaskCallback callback;
    void *callbackArg;

    union {
        ES_DSP_TASK_S dspTask;
        esHaeTask_t haeTask;
        int16_t eventId;
    } opTaskPrivate;
} opPrivate_t;

typedef struct _opTaskCommDesc {
    uint8_t opType;
    uint8_t opStatus;
    int16_t opIndex;
    const dataDesc_t *UmdDataDesc;
    class TaskScheduler *taskScheduler;
} opTaskCommDesc_t;

typedef struct _opTaskDesc {
    opTaskCommDesc_t commDesc;
    opPrivate_t opPrivate;
    bool isDump;
} opTaskDesc_t;

struct _npuTask;
// 这个数据结构的长度不能超过sdkPrivate数组的长度
typedef struct _taskPrivate {
    struct list_head node;
    void *stream;
    std::vector<struct _npuTask *> tasks;
    int64_t submitClock;
    uint32_t userModelId;
    bool isFlexibleTask;
} taskPrivate_t;

static_assert(sizeof(taskPrivate_t) <= ES_TASK_SDK_PRIVATE_LEN, "the size of taskPrivate_t is too large!");

// 这是runtime内部使用的task数据结构
typedef struct _npuTask {
    uint32_t taskId;
    uint32_t modelId;
    uint8_t inputFdNum;
    uint8_t outputFdNum;
    ES_DEV_BUF_S inputFd[ES_TASK_MAX_FD_CNT];
    ES_DEV_BUF_S outputFd[ES_TASK_MAX_FD_CNT];
    NPU_TaskCallback callback;
    void *callbackArg;
    NPU_TASK_STATE_E state;
    taskPrivate_t priv;
    uint8_t reserved[ES_TASK_SDK_PRIVATE_LEN - sizeof(taskPrivate_t)];
} npuTask_t;

static_assert(sizeof(npuTask_t) == sizeof(NPU_TASK_S), "sizeof(taskPrivate_t) not equal sizeof(NPU_TASK_S)!");

}  // namespace eswin

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_TASK_H_
