// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#include <filesystem>
#include <sstream>
#include "npu_test_utils.h"

std::vector<std::string> listSubDirectory(std::string dir)
{
    std::vector<std::string> result;

    for (const auto &entry : std::filesystem::directory_iterator(dir)) {
        result.push_back(entry.path().string());
    }

    return result;
}

int32_t extractModelFiles(std::vector<std::string> &modelDirs, std::vector<std::string> &modelFiles)
{
    for (auto &dir : modelDirs) {
        std::string filePath;
        std::string fileName;
        for (const auto &entry : std::filesystem::directory_iterator(dir)) {
            if (entry.is_regular_file() && entry.path().extension() == ".model") {
                filePath = entry.path().string();
                fileName = entry.path().filename().string();
                if (fileName.find("npu") != std::string::npos) break;
            }
        }
        if (!filePath.empty()) {
            modelFiles.push_back(filePath);
        } else {
            printf("No model file found in %s\n", dir.c_str());
            return -1;
        }
    }

    return 0;
}

std::string extractConfigFiles(std::string &configDir, std::string pattern)
{
    std::string filePath;

    for (const auto &entry : std::filesystem::directory_iterator(configDir)) {
        if (entry.is_regular_file() && entry.path().filename().string().find(pattern) != std::string::npos) {
            filePath = entry.path().string();
            break;
        }
    }

    return filePath;
}

std::vector<std::string> split(std::string input, char delimiter)
{
    std::vector<std::string> result;
    std::string item;
    std::stringstream ss(input);

    while (std::getline(ss, item, delimiter)) {
        result.push_back(item);
    }

    return result;
}