// Copyright © 2024 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef _PREPROCESS_H
#define _PREPROCESS_H

#include <stdint.h>
#include <string>
#include <vector>
#include <mutex>

#define DEBUG_MODE 0
#if DEBUG_MODE
#define DEBUG_INFO(fmt, ...) printf("INFO: " fmt "", ##__VA_ARGS__)
#define DEBUG_ERROR(fmt, ...) printf("ERROR: " fmt "", ##__VA_ARGS__)
#else
#define DEBUG_INFO(fmt, ...)
#define DEBUG_ERROR(fmt, ...) printf("ERROR: " fmt "", ##__VA_ARGS__)
#endif

class EsPreProcess
{
 public:
    EsPreProcess();
    virtual ~EsPreProcess();

 public:
    virtual bool initializeFromJson(uint32_t devId, const std::string &cfgFilePath);
    virtual int32_t preprocess(const std::string &jpegPath, uint64_t fd, uint64_t offset, uint64_t size);
    virtual void release();

 private:
    virtual int32_t preprocessInternal(const std::string &jpegPath, uint8_t *inputBuf, int32_t size);

 protected:
    double mMeanValues[3];
    double mStdValues[3];
    int mResizeShape;
    int mCropWidth;
    int mCropHeight;
    double mQuantizationScale;
    bool mKeepRatio;
    bool mChannelFirst;
    bool mNorm;
    std::string mInputFormat;
    std::mutex mMtx;
};

#endif  // _PREPROCESS_H
